<?php
/**
 * Copyright ©  All rights reserved.
 * See COPYING.txt for license details.
 */
declare(strict_types=1);

namespace Pinpoint\Conesso\Cron;

use Magento\Quote\Model\Quote\Item;
use Magento\Quote\Model\QuoteRepository;
use Pinpoint\Conesso\Api\ConfigConstantsInterface;
use Pinpoint\Conesso\Model\Api\Service as ConessoService;
use Psr\Log\LoggerInterface;
use Magento\Quote\Model\ResourceModel\Quote\CollectionFactory as QuoteCollectionFactory;
use Magento\Quote\Model\ResourceModel\Quote\Collection as QuoteCollection;
use Magento\Quote\Model\Quote;
use Magento\Framework\App\Config\ScopeConfigInterface;

class AbandonedCarts
{
    /**
     * @param LoggerInterface $logger
     * @param ConessoService $service
     * @param QuoteCollectionFactory $quoteCollectionFactory
     * @param QuoteRepository $quoteRepository
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        private readonly LoggerInterface $logger,
        private readonly ConessoService $service,
        private readonly QuoteCollectionFactory $quoteCollectionFactory,
        private readonly QuoteRepository $quoteRepository,
        private readonly ScopeConfigInterface $scopeConfig
    ) {
        //__construct
    }

    /**
     * Execute the cron
     *
     * @return void
     */
    public function execute(): void
    {
        $enabled = $this->scopeConfig->getValue(ConfigConstantsInterface::XML_PATH_CONESSO_ABANDONED_CART_ENABLED);

        if (!$enabled) {
            return;
        }

        $beginFrom = $this->scopeConfig->getValue(ConfigConstantsInterface::XML_PATH_CONESSO_ABANDONED_CART_BEGIN_FROM);
        $triggerInterval = $this->scopeConfig->getValue(ConfigConstantsInterface::XML_PATH_CONESSO_ABANDONED_CART_TRIGGER_INTERVAL);
        $triggerTime = strtotime('-' . $triggerInterval . ' hours');

        /** @var QuoteCollection $abandonedQuoteCollection */
        $abandonedQuoteCollection = $this->quoteCollectionFactory->create();
        $abandonedQuoteCollection
            ->addFieldToFilter('is_active', 1)
            ->addFieldToFilter('created_at', ['gteq' => $beginFrom])
            ->addFieldToFilter('conesso_ac_triggered', 0);


        /** @var Quote $quote */
        foreach ($abandonedQuoteCollection as $quote) {
            $updateAt = $quote->getUpdatedAt();
            if (strtotime($updateAt) <= $triggerTime) {
                $this->service->carts(
                    ['json' => $this->convertQuoteToConesso($quote)],
                    'POST'
                );

                $quote->setData('conesso_ac_triggered', 1);

                try {
                    $this->quoteRepository->save($quote);
                } catch (\Exception $e) {
                    $this->logger->error($e->getMessage());
                }
            }
        }
    }

    /**
     * @param Quote $quote
     * @return array
     */
    private function convertQuoteToConesso(Quote $quote): array
    {
        $baseSubtotalWithDiscount = $quote->getBaseSubtotalWithDiscount();
        $baseSubtotal = $quote->getBaseSubtotal();
        $grandTotalDiscount = $baseSubtotal - $baseSubtotalWithDiscount;

        $cartProducts = array_map(function (Item $item) {
            return [
                'productSku' => $item->getSku(),
                'qty' => $item->getQty(),
                'cartLineId' => $item->getItemId(),
                'productName' => $item->getName(),
            ];
        }, $quote->getAllVisibleItems());

        $createdAtUTC = date('Y-m-d\TH:i:s\Z', strtotime($quote->getCreatedAt()));
        $updatedAtUTC = date('Y-m-d\TH:i:s\Z', strtotime($quote->getUpdatedAt()));

        return [
            'contactId' => 0,
            'customerEmail' => $quote->getCustomerEmail(),
            'apiReferenceId' => str_pad($quote->getId(), 5, '0', STR_PAD_LEFT),
            'apiIntegration' => 'Magento2',
            'apiResource' => 'carts',
            'confirmedIsAbandoned' => true,
            'isActive' => 'false',
            'total' => $quote->getSubtotal() * 100,
            'totalIncTax' => $quote->getGrandTotal() * 100,
            'isoCurrencyCode' => $quote->getQuoteCurrencyCode(),
            'abandonedTotal' => $quote->getGrandTotal() * 100,
            'abandonedTotalWithDiscount' => $quote->getGrandTotal() * 100,
            'abandonedTotalDiscount' => $grandTotalDiscount * 100,
            'customerFirstname' => $quote->getCustomerFirstname(),
            'customerGender' => '',
            'customerLastname' => $quote->getCustomerLastname(),
            'customerTelephone' => $quote->getBillingAddress()->getTelephone(),
            'customerMobile' => '',
            'customerDob' => $quote->getCustomerDob(),
            'customerNotes' => '',
            'billingFirstname' => $quote->getBillingAddress()->getFirstname(),
            'billingLastname' => $quote->getBillingAddress()->getLastname(),
            'billingAddress1' => $quote->getBillingAddress()->getStreetLine(1),
            'billingAddress2' => $quote->getBillingAddress()->getStreetLine(2),
            'billingCity' => $quote->getBillingAddress()->getCity(),
            'billingCounty' => $quote->getBillingAddress()->getRegion(),
            'billingPostcode' => $quote->getBillingAddress()->getPostcode(),
            'billingCountry' => $quote->getBillingAddress()->getCountryId(),
            'shippingFirstname' => $quote->getShippingAddress()->getFirstname(),
            'shippingLastname' => $quote->getShippingAddress()->getLastname(),
            'shippingAddress1' => $quote->getShippingAddress()->getStreetLine(1),
            'shippingAddress2' => $quote->getShippingAddress()->getStreetLine(2),
            'shippingCity' => $quote->getShippingAddress()->getCity(),
            'shippingCounty' => $quote->getShippingAddress()->getRegion(),
            'shippingCountry' => $quote->getShippingAddress()->getCountryId(),
            'shippingPostcode' => $quote->getShippingAddress()->getPostcode(),
            'cartCreatedAt' => $createdAtUTC,
            'cartUpdatedAt' => $updatedAtUTC,
            'cartProducts' => $cartProducts,
            'customerIsGuest' => (int) $quote->getCustomerIsGuest(),
        ];
    }
}
