<?php
namespace Pinpoint\Conesso\Model\Api\Management\Transfer\Request;

use Exception;
use Magento\Quote\Model\Quote\Item;
use Magento\Quote\Model\ResourceModel\Quote\CollectionFactory;
use Magento\Framework\DataObject;
use Pinpoint\Conesso\Api\Data\CartSyncInterface;
use Pinpoint\Conesso\Api\Repository\CartsSyncRepositoryInterface;
use Pinpoint\Conesso\Model\Api\Access\Auth;
use Pinpoint\Conesso\Model\Config\CartSettings;

class CartRequest extends AbstractRequest
{
    /**
     * @var CartsSyncRepositoryInterface
     */
    protected CartsSyncRepositoryInterface $cartsSyncRepository;

    /**
     * @var CollectionFactory
     */
    protected CollectionFactory $cartCollectionFactory;

    /**
     * @var CartSettings
     */
    private CartSettings $cartSettings;

    /**
     * @var CustomerRequest
     */
    private CustomerRequest $customerRequest;

    /**
     * @var ProductRequest
     */
    private ProductRequest $productRequest;

    /**
     * @param Auth $authorization
     * @param CartsSyncRepositoryInterface $cartsSyncRepository
     * @param CollectionFactory $cartCollectionFactory
     * @param CartSettings $cartSettings
     * @param CustomerRequest $customerRequest
     * @param ProductRequest $productRequest
     */
    public function __construct(
        Auth $authorization,
        CartsSyncRepositoryInterface $cartsSyncRepository,
        CollectionFactory $cartCollectionFactory,
        CartSettings $cartSettings,
        CustomerRequest $customerRequest,
        ProductRequest $productRequest
    ) {
        $this->cartsSyncRepository = $cartsSyncRepository;
        $this->cartCollectionFactory = $cartCollectionFactory;
        $this->cartSettings = $cartSettings;
        $this->customerRequest = $customerRequest;
        $this->productRequest = $productRequest;
        parent::__construct($authorization);
    }

    /**
     * @param mixed $quote
     * @return array
     */
    public function buildData(mixed $quote): array
    {
        $baseSubtotalWithDiscount = $quote->getBaseSubtotalWithDiscount();
        $baseSubtotal = $quote->getBaseSubtotal();
        $grandTotalDiscount = $baseSubtotal - $baseSubtotalWithDiscount;

        $cartProducts = array_map(function (Item $item) use ($quote) {
            return [
                'productSku' => $item->getSku(),
                'qty' => $item->getQty(),
                'cartLineId' => $item->getItemId(),
                'productName' => $item->getName(),
            ];
        }, $quote->getAllVisibleItems());

        $createdAtUTC = date('Y-m-d\TH:i:s\Z', strtotime($quote->getCreatedAt()));
        $updatedAtUTC = date('Y-m-d\TH:i:s\Z', strtotime($quote->getUpdatedAt()));

        return [
            'customerEmail' => $quote->getCustomerEmail() ?? 'guest1@idhl.co.uk',
            'apiReferenceId' => str_pad($quote->getId(), 5, '0', STR_PAD_LEFT),
            'apiIntegration' => 'Magento2',
            'apiResource' => 'carts',
            'confirmedIsAbandoned' => false,
            'isActive' => 'false',
            'total' => $quote->getSubtotal() * 100,
            'totalIncTax' => $quote->getGrandTotal() * 100,
            'isoCurrencyCode' => $quote->getQuoteCurrencyCode(),
            'abandonedTotal' => $quote->getGrandTotal() * 100,
            'abandonedTotalWithDiscount' => $quote->getGrandTotal() * 100,
            'abandonedTotalDiscount' => $grandTotalDiscount * 100,
            'customerFirstname' => $quote->getCustomerFirstname(),
            'customerGender' => '',
            'customerLastname' => $quote->getCustomerLastname(),
            'customerTelephone' => $quote->getBillingAddress()->getTelephone(),
            'customerMobile' => '',
            'customerDob' => $quote->getCustomerDob(),
            'customerNotes' => '',
            'billingFirstname' => $quote->getBillingAddress()->getFirstname(),
            'billingLastname' => $quote->getBillingAddress()->getLastname(),
            'billingAddress1' => $quote->getBillingAddress()->getStreetLine(1),
            'billingAddress2' => $quote->getBillingAddress()->getStreetLine(2),
            'billingCity' => $quote->getBillingAddress()->getCity(),
            'billingCounty' => $quote->getBillingAddress()->getRegion(),
            'billingPostcode' => $quote->getBillingAddress()->getPostcode(),
            'billingCountry' => $quote->getBillingAddress()->getCountryId(),
            'shippingFirstname' => $quote->getShippingAddress()->getFirstname(),
            'shippingLastname' => $quote->getShippingAddress()->getLastname(),
            'shippingAddress1' => $quote->getShippingAddress()->getStreetLine(1),
            'shippingAddress2' => $quote->getShippingAddress()->getStreetLine(2),
            'shippingCity' => $quote->getShippingAddress()->getCity(),
            'shippingCounty' => $quote->getShippingAddress()->getRegion(),
            'shippingCountry' => $quote->getShippingAddress()->getCountryId(),
            'shippingPostcode' => $quote->getShippingAddress()->getPostcode(),
            'cartCreatedAt' => $createdAtUTC,
            'cartUpdatedAt' => $updatedAtUTC,
            'cartProducts' => $cartProducts,
            'customerIsGuest' => (int) $quote->getCustomerIsGuest(),
        ];
    }

    /**
     * @param mixed $data
     * @return bool
     */
    public function sendRequest(mixed $data): bool
    {
        foreach ($data['items'] as $item) {
            if ($item->getEventType() == CartSyncInterface::EVENT_TYPE_SAVE) {
                $this->saveCart($item);
            }
            $this->updatedSynchronizedData($item);
        }
        return true;
    }

    /**
     * @param $item
     * @return void
     */
    private function updatedSynchronizedData($item): void
    {
        $item->setIsSynced(1);
        $this->cartsSyncRepository->save($item);
    }

    /**
     * @param $item
     * @return void
     */
    private function saveCart($item): void
    {
        if ($cart = $this->getCartById($item->getCartId())) {
            $parameters = $this->buildData($cart);
            $apiReference = str_pad($cart->getId(), 5, '0', STR_PAD_LEFT);
            if($apiReference && $cartId = $this->getCartIdInConesso($apiReference)) {
                $this->updateCartInConesso($cartId, $parameters);
            } else {
                $this->getConnessoClient()->carts()->create($parameters);
            }
        }
    }

    /**
     * @param $apiReference
     * @return false|mixed
     */
    private function getCartIdInConesso($apiReference): mixed
    {
        try {
            $carts = $this->getConnessoClient()->carts()->list()->data;
            foreach ($carts as $cart) {
                if ($cart['apiReferenceId'] == $apiReference) {
                    return $cart['id'];
                }
            }
            return false;
        } catch (TypeError|Exception $e) {
            return false;
        }
    }


    /**
     * @param $cartId
     * @param $parameters
     * @return void
     */
    private function updateCartInConesso($cartId, $parameters): void
    {
        $this->getConnessoClient()->carts()->update($cartId, $parameters);
    }

    /**
     * @param $customerId
     * @return false|DataObject
     */
    private function getCartById($customerId): DataObject|bool
    {
        try {
            return $this->cartCollectionFactory->create()->addFieldToFilter('entity_id', $customerId)->getFirstItem();
        } catch (Exception $e) {
            return  false;
        }
    }

    /**
     * @param $customerEmail
     * @return false|mixed
     */
    private function getContactIdByEmail($customerEmail): mixed
    {
        return $this->customerRequest->existsContactConesso($customerEmail);
    }
}
