<?php
namespace Pinpoint\Conesso\Model\Api\Management\Transfer\Request;

use Exception;
use Magento\Customer\Model\ResourceModel\Customer\CollectionFactory;
use Magento\Framework\DataObject;
use Pinpoint\Conesso\Api\Data\ContactSyncInterface;
use Pinpoint\Conesso\Api\Repository\ContactsRepositoryInterface;
use Pinpoint\Conesso\Model\Api\Access\Auth;
use Pinpoint\Conesso\Model\Config\ContactSettings;

class CustomerRequest extends AbstractRequest
{
    /**
     * @var ContactsRepositoryInterface
     */
    protected ContactsRepositoryInterface $contactsRepository;

    /**
     * @var CollectionFactory
     */
    protected CollectionFactory $customerCollectionFactory;

    /**
     * @var ContactSettings
     */
    private ContactSettings $contactSettings;

    /**
     * @param Auth $authorization
     * @param ContactsRepositoryInterface $contactsRepository
     * @param CollectionFactory $customerCollectionFactory
     * @param ContactSettings $contactSettings
     */
    public function __construct(
        Auth $authorization,
        ContactsRepositoryInterface $contactsRepository,
        CollectionFactory $customerCollectionFactory,
        ContactSettings $contactSettings
    ) {
        $this->contactsRepository = $contactsRepository;
        $this->customerCollectionFactory = $customerCollectionFactory;
        $this->contactSettings = $contactSettings;
        parent::__construct($authorization);
    }

    /**
     * @param mixed $customer
     * @return array
     */
    public function buildData(mixed $customer): array
    {
        $data = [
            "email" => $customer->getEmail(),
            "firstName" => $customer->getFirstName(),
            "lastName" => $customer->getLastName(),
            "optInStatus" => "subscribed"
        ];
        return array_merge($data, $this->getContactCustomAttributes($customer));
    }

    /**
     * @param $customer
     * @return array
     */
    private function getContactCustomAttributes($customer): array
    {
        $data = [];
        $customAttributes = $this->contactSettings->getCustomAttributes();
        if($customAttributes) {
            foreach ($customAttributes as $key => $row) {
                $conessoAttribute = $row['conesso_attribute'];
                $magentoAttribute = $row['magento_attribute'];
                if($conessoAttribute && $magentoAttribute) {
                    $data[$conessoAttribute] = $customer->getData($magentoAttribute);
                }
            }
        }

        return $data;
    }

    /**
     * @param mixed $data
     * @return bool
     */
    public function sendRequest(mixed $data): bool
    {
        foreach ($data['items'] as $item) {
            if ($item->getEventType() == ContactSyncInterface::EVENT_TYPE_SAVE) {
                $this->saveCustomer($item);
            }
            if ($item->getEventType() == ContactSyncInterface::EVENT_TYPE_DELETE)
            {
                $this->deleteCustomerInConesso($item);
            }
            $this->updatedSynchronizedData($item);
        }
        return true;
    }

    /**
     * @param $item
     * @return void
     */
    private function updatedSynchronizedData($item): void
    {
        $item->setIsSynced(1);
        $this->contactsRepository->save($item);
    }

    /**
     * @param $item
     * @return void
     */
    private function saveCustomer($item): void
    {
        if ($customer = $this->getCustomerById($item->getCustomerId())) {
            $parameters = $this->buildData($customer);
            $customerEmail = $customer->getEmail();
            if($contactId = $this->existsContactConesso($customerEmail)) {
                $this->updateContactInConesso($contactId, $parameters);
            } else {
                $this->getConnessoClient()->contacts()->create($parameters);
            }
        }

    }

    /**
     * @param $customerEmail
     * @return false|mixed
     */
    public function existsContactConesso($customerEmail): mixed
    {
        try {
            $contacts = $this->getConnessoClient()->contacts()->list()->data;

            foreach ($contacts as $contact) {
                if ($contact['email'] == $customerEmail) {
                    return $contact['id'];
                }
            }
            return false;
        } catch (TypeError|Exception $e) {
            return false;
        }
    }

    /**
     * @param $contactId
     * @param $parameters
     * @return void
     */
    private function updateContactInConesso($contactId, $parameters): void
    {
        $this->getConnessoClient()->contacts()->update($contactId, $parameters);
    }

    /**
     * @param $customerId
     * @return false|DataObject
     */
    private function getCustomerById($customerId): DataObject|bool
    {
        try {
            return $this->customerCollectionFactory->create()->addFieldToFilter('entity_id', $customerId)->getFirstItem();
        } catch (Exception $e) {
            return  false;
        }
    }

    /**
     * @param $item
     * @return void
     */
    private function deleteCustomerInConesso($item): void
    {
        $contactId = $this->existsContactConesso($item->getEmail());
        $this->getConnessoClient()->contacts()->delete($contactId);
    }
}
